{*************************************
 *  Hydra- NES  Emulator Version 1.0 *
 *************************************

Copyright (C) 2010 Darryl Biggar

Version of 13 February 2010

This program primarily starts all the relevant cogs. It also contains debugging code for monitoring the
status and progress of the debugger.
}

CON

  _clkmode = xtal1 + pll8x  ' Set clock multiplier to 8x
  _xinfreq = 10_000_000 + 0_000 ' Set to 10Mhz (and add 5000 to fix crystal imperfection of hydra prototype)
  _free = 159

  SCANLINE_BUFFER = $7F00

  spin_ppuctrl_adr = SCANLINE_BUFFER-16                 ' $7EF0
  spin_ppumask_adr = SCANLINE_BUFFER-15                 ' $7EF1
  spin_ppustatus_adr=SCANLINE_BUFFER-14                 ' $7EF2
  spin_OAMaddr_adr = SCANLINE_BUFFER-13                 ' $7EF3
  spin_horizontal_scroll = SCANLINE_BUFFER-11           ' $7EF5
  spin_vertical_scroll = SCANLINE_BUFFER-12             ' $7EF4
  spin_spritetable_adr = SCANLINE_BUFFER-10             ' $7EF6
  spin_nametables_adr = SCANLINE_BUFFER-8               ' $7EF8  
  spin_pallettes_adr = SCANLINE_BUFFER-6                ' $7EFA
  spin_patterntables_adr = SCANLINE_BUFFER-4            ' $7EFC
  spin_request_scanline = SCANLINE_BUFFER-2             ' $7EFE
  spin_nmi_request_adr = SCANLINE_BUFFER-17             ' $7EEF

' Gamepad constant declaration
  NES_RIGHT  = %00000001
  NES_LEFT   = %00000010
  NES_DOWN   = %00000100
  NES_UP     = %00001000
  NES_START  = %00010000
  NES_SELECT = %00100000
  NES_B      = %01000000
  NES_A      = %10000000


VAR

  long tv_status      '0/1/2 = off/visible/invisible           read-only

  long  temp1,temp2

  long  cog_number     ' Current rendering cog ID
  long  cog_total      ' Total number of required rendering cog
  long  horiz_offset
  long  vert_offset

  long  emu_mode                ' these variables have to be in this order
  long  emu_status
  long  reg6502_acc
  long  reg6502_xreg
  long  reg6502_yreg
  long  reg6502_status
  long  reg6502_stackptr
  long  reg6502_pc
  long  memory_offset
  long  memory_start
  
  long  letter_offset
  long  number_offset
  long  nt,ms
  long  dnt           ' name table to use for display
  long  button_pressed
  long  bp_count,blank_char,dash_char
  long  char_pattern_table,Start_of_PRG_ROM

                                                                                                       
OBJ


  emu   : "NES_emulator_core_010.spin"
  tv    : "NES_tv_017.spin"
  ppu   : "NES_PPU_driver_011.spin"
'  data  : "NES_Galaga_PPUrom_data_002.spin"
'  rom   : "NES_Galaga_PRGrom_data_001.spin"
  data  : "NES_Pacman_PPUrom_data_010.spin"
  rom   : "NES_Pacman_PRGrom_data_011.spin"
'  data  : "NES_DonkeyKong_PPUrom_data_001.spin"
'  rom   : "NES_DonkeyKong_PRGrom_data_001.spin"


PUB start

  ' Set up pointers to data tables:
'    nt := @name_tables
  ms := emu.start_of_emulator   
  nt := ms+$0800 

' For Donkey Kong uncomment the following
'
{
    letter_offset := 9  
    number_offset := 0  
    dnt :=nt ' +$0400   
    blank_char := $24
    dash_char :=  $3A
    char_pattern_table := 1          ' 0 for first pattern table, 1 for second
    Start_of_PRG_ROM := $C000      ' 
}

' For Galaga uncomment the following
'
{
    letter_offset := 9 
    number_offset := 0
    dnt :=nt +$0400                ' for Galaga better to display on second name table
    blank_char := $24
    dash_char :=  $27
    char_pattern_table := 1          ' 0 for first pattern table, 1 for second
    Start_of_PRG_ROM := $BEA0      ' Galaga ROM starts at $BEA0 ($BE00 but I deleted unused mem)
}
'
'
' for Pacman uncomment the following
'
'{
    letter_offset := 64 
    number_offset := 48
    dnt :=nt ' +$0400
    blank_char := $20
    dash_char :=  58
    char_pattern_table := 0          ' 0 for first pattern table, 1 for second
    Start_of_PRG_ROM := $C000
'}

  word[spin_pallettes_adr] := data.pallettes
  word[spin_nametables_adr] := nt
  word[spin_patterntables_adr] := data.pattern_tables
  word[spin_spritetable_adr] := data.sprite_data
    
  ' Set up PPU initial conditions
  byte[spin_ppuctrl_adr]    := %0001_0000            ' start with the second pattern table
  byte[spin_ppumask_adr]    := %0001_1010
  byte[spin_vertical_scroll] :=0
  byte[spin_horizontal_scroll] := 0
  byte[spin_nmi_request_adr] := 0
  
  ' Setup initial conditions for 6502 registers

' emu_mode = 0 - do nothing
' emu_mode = 1 - step-by-step mode
' emu_mode = 2 - run full speed
' emu_mode = 3 - run continuously, writing out each step

emu_mode := 2                                    ' start off running
bp_count := 0                                    ' stop after n breakpoints reached
{  emu_status := 0               
  reg6502_acc := $FF
  reg6502_xreg := $FF
  reg6502_yreg := $FF
  reg6502_status := $FF}
  reg6502_stackptr := $FF
  memory_offset := rom.NES_memory_start-Start_of_PRG_ROM   ' amount to add to get ROM data
  memory_start := ms                            ' amount to add to get zero page addresses
  reg6502_pc := word[$FFFC+memory_offset]       ' set to RESET address in PPU rom
  

  ' Boot rendering cogs:

  cog_total := 5
  cog_number := 0
  repeat
    ppu.start(@cog_number)
    repeat 10000 ' Allow some time for previous cog to boot up before setting 'cog_number' again
    cog_number++
  until cog_number == cog_total


  tv.start(@tv_status)
  emu.start(@emu_mode)

  bytefill(nt,blank_char,2048)          ' clear name tables to blank screen
  bytefill(nt+960,0,64)                                 ' clear attribute table 1
  bytefill(nt+960+$0400,0,64)                           ' clear attribute table 2
  
  byte[memory_start+ $0130] := 3                  ' Seems necessary to fix a bug in the original Pacman program??   
  byte[memory_start+ $0131] := $FF                ' Seems necessary to fix a bug in the original Pacman program??   
        
  button_pressed := False

  repeat
    ' Wait for a VBL using the legacy TV variable 'tv_status'
    repeat while tv_status == 1
    repeat while tv_status == 2
'{
    ' Read both gamepad
    if emu_mode==0
       if bp_count>0
          bp_count--
          emu_mode := 2
       temp1 := NES_Read_Gamepad
       byte[spin_ppuctrl_adr]    := ((temp1 >> 4)& 3) | (char_pattern_table << 4 )
       word[spin_vertical_scroll] :=0

    if (temp1 & NES_LEFT)         ' Left button - run while button is held
'        byte[spin_ppuctrl_adr] := $90
'        byte[spin_vblank_adr] := 0
'        emu_mode :=2
       emu_mode := 1

    if button_pressed
      if (temp1 == 0)
        button_pressed := False
    if not button_pressed
      if (temp1 & NES_RIGHT)
        button_pressed := True     ' Right button - run one step at a time
        emu_mode :=1
      if (temp1 & NES_A)
        button_pressed := True
        emu_mode :=3
'        running_to_breakpoint := True
      if (temp1 & NES_B)
        button_pressed := True     ' B Button - run at full speed
'        byte[spin_ppuctrl_adr] := $90
'        byte[spin_vblank_adr] := 0
        emu_mode :=2

    if emu_mode<>2 and bp_count==0
       Show_Emulation_Info
       Hex(0,0,$2000,4)
'       Hex($400,0,$2800,4)
'       Hex($800,0,$2800,4)
'       Hex($C00,0,$2C00,4)
       Show_NES_RAM_Content($0,ms)    ' Shows zero page of NES RAM
'       Show_NES_RAM_Content($FF80,memory_offset)    ' Shows NES ROM                                             E
'       Show_NES_RAM_Content($2080,nt-$2000)' Shows PPU name table RAM
'    Show_NES_RAM_Content(SCANLINE_BUFFER-32,0) ' data.pallettes,0)                  ' Shows Hydra HUB memory
'    Show_NES_RAM_Content(@emu_mode,0) ' data.pallettes,0)                  ' Shows Hydra HUB memory
'       Show_NES_RAM_Content($0000,data.pattern_tables)
'        Show_NES_RAM_Content($3F00,data.pallettes-$3F00) ' Shows PPU pallette memory

'      

PUB Show_Emulation_Info

    Hex(112,24,reg6502_xreg,2)  ' X-xx
    Hex(117,25,reg6502_yreg,2)  ' Y-xx
    Hex(122,19,reg6502_stackptr,2)  ' S-xx
    byte[dnt+136] := letter_offset+16 ' place "P"
'    Hex(169,1,byte[$430+memory_start],2)     ' PC-xxxx
'    Hex(169,1,$430+memory_start,4)     ' PC-xxxx
    byte[dnt+144] := letter_offset+14 ' "N"
    byte[dnt+145] := letter_offset+22 ' "V"
    byte[dnt+146] := letter_offset+21 ' "unused"
    byte[dnt+147] := letter_offset+2  ' "B"
    byte[dnt+148] := letter_offset+4  ' "D"
    byte[dnt+149] := letter_offset+9  ' "I"
    byte[dnt+150] := letter_offset+26 ' "Z"
    byte[dnt+151] := letter_offset+3  ' "C"
    
    Binary(176,0,reg6502_status,8)
    Hex(137,3,reg6502_pc,4)     ' PC-xxxx
    Hex(105,1,reg6502_acc,2)    ' A-xx
    Hex(186,5,emu_status,1)
 
    if  (reg6502_pc & $8000)
      temp2 := reg6502_pc+memory_offset
    else
      temp2 := reg6502_pc+memory_start
      
    Hex(96,0,reg6502_pc,4)
    Hex(101,0,byte[temp2],2)
    Hex(128,0,reg6502_pc+1,4)
    Hex(133,0,byte[temp2+1],2)
    Hex(160,0,reg6502_pc+2,4)
    Hex(165,0,byte[temp2+2],2)
                        
PUB Show_NES_RAM_Content(mem_loc,offset)

    temp2 :=0
    repeat 4 ' 8 ' 16
      Hex(224+((temp2)>>3)<<5,0,mem_loc,4)       ' was 224+
      repeat 8
        Hex(229+(((temp2)>>3)<<5)+(temp2&7)*3,0,byte[offset+mem_loc],2)
        mem_loc++
        temp2++
        
 
        
'---------------------------------------------

' General read gamepad spin function                       
PUB NES_Read_Gamepad : nes_bits   |       i
  DIRA [3] := 1 ' output
  DIRA [4] := 1 ' output
  DIRA [5] := 0 ' input
  DIRA [6] := 0 ' input

  OUTA [3] := 0 ' JOY_CLK = 0
  OUTA [4] := 0 ' JOY_SH/LDn = 0
  OUTA [4] := 1 ' JOY_SH/LDn = 1
  OUTA [4] := 0 ' JOY_SH/LDn = 0
  nes_bits := 0
  nes_bits := INA[5] | (INA[6] << 8)

  repeat i from 0 to 6
    OUTA [3] := 1 ' JOY_CLK = 1
    OUTA [3] := 0 ' JOY_CLK = 0
    nes_bits := (nes_bits << 1)
    nes_bits := nes_bits | INA[5] | (INA[6] << 8)

  nes_bits := (!nes_bits & $FFFF)

PUB Hex(loc,letter,amount,digits)

  if letter>0
    byte[dnt+loc] := letter_offset+letter
    byte[dnt+loc+1] := dash_char ' $27 is the "-" character in the Galaga nametable
    loc+=2
  loc := loc+digits-1
  repeat while digits>0
    temp1 := (amount & 15)
    if temp1>9
      byte[dnt+loc] := letter_offset+temp1-9
    else
      byte[dnt+loc] := number_offset+temp1
    amount >>= 4
    loc--
    digits--
    

PUB Binary(loc,letter,amount,digits)

  if letter>0
    byte[dnt+loc] := letter_offset+letter
    byte[dnt+loc+1] := dash_char ' "-"
    loc+=2
  loc := loc+digits-1
  repeat while digits>0
    temp1 := (amount & 1)
    byte[dnt+loc] := number_offset+temp1
    amount >>= 1
    loc--
    digits--                                                      